# Simple RADIUS service for coupling Juniper/Netscreen firewalls to ADS.
#
# (C) 2006-2007 by Marc Huber <Marc.Huber@web.de>
#
# Based upon sample code from the various modules used, especially
# - RADIUS::Dictionary/::Packet by Christopher Masto <chris@netmonger.net>
# - Win32::Daemon by Dave Roth <rothd@roth.net>
#
# The dictionary data used comes from the FreeRadius distribution
# (http://www.freeradius.org).
#

#
# This work is licensed to you under the terms of
# "The GNU General Public License, version 2".
#

#
# Set tab stops to 4 if the formatting looks weird, or use vim, which will
# handle this automatically.
#

# Possibly incomplete list of requirements:
# - ActivePerl
# - Perl modules/PPMs:
#  - RADIUS module (http://search.cpan.org/dist/RADIUS-1.0/)
#  - perl-ldap module (http://search.cpan.org/dist/perl-ldap-0.33/)
#  - Win32::Daemon module
#    (ppm install http://www.roth.net/perl/packages/win32-daemon.ppd)
# - "config.txt" file in same directory as main script

# Configuration file syntax -- please note that configuration
# is CASE SENSITIVE.
#
#
# ADS				<ip>		# ADS IP Address for LDAP bind
# 								# default: 127.0.0.1:389
# 								# (optional, may appear multiple times)
#
# DOMAIN			<string>	# DNS domain to use for user authentication
# 								# If omitted, users will have to use
# 								# "user@domain" syntax.
# 								# (optional, may appear multiple times)
#
# PORT				<number>	# UDP port for receiving RADIUS authentication
# 								# and/or accounting packets
# 								# defaults: 1812, 1813
# 								# (optional, may appear multiple times)
#
# NS-Primary-DNS	<ip>		# Primary DNS server for VPN users
# 								# (optional, may appear only once)
#
# NS-Secondary-DNS	<ip>		# Secondary DNS server for VPN users
# 								# (optional, may appear only once)
#
# NS-Primary-WINS	<ip>		# Primary WINS server for VPN users
# 								# (optional, may appear only once)
#
# NS-Secondary-WINS	<ip>		# Secondary WINS server for VPN users
# 								# (optional, may appear only once)
#
# CLIENT	<ip>	<secret>	# RADIUS shared secret for client IP address
# 								# (optional, may appear multiple times)
#
# GROUP-REGEX		<regex>		# Perl regular expression a group DN needs to
# 								# match.
# 								# (optional, may appear multiple times)
#

#
# Sample configuration:
#
# ADS 172.16.0.129
# DOMAIN example.org
# DOMAIN example.com
# NS-Primary-DNS 1.2.3.4
# CLIENT 172.16.0.100 mysecret
# CLIENT 172.16.0.1 mysecret
# GROUP-REGEX ^cn=[0-9a-z]+,
#

#
# Minumum configuration:
#
# DOMAIN example.com
# CLIENT 172.16.0.254 mysecret
#

use strict;
use RADIUS::Dictionary;
use RADIUS::Packet;
use Net::LDAP;
use Win32::Daemon;
use IO::Select;
use IO::Socket::INET;

my %CONFIG;
my %secret;

my $debug = undef;

$0 =~ /^(.*\\)[^\\]+$/;
chdir $1 or die "chdir $1: $!";
#open STDERR, ">error.txt";

open C, 'config.txt' or die "Couldn't read configuration: $!";
while (<C>){
	s/#.*$//g;
	if (/^\s*([^\s]+)\s+([^\s]+)\s*$/){
		$CONFIG{$1} = [] unless exists $CONFIG{$1};
		push (@{$CONFIG{$1}}, $2);
	} elsif (/^\s*CLIENT\s+([^\s]+)\s+([^\s]+)\s*$/){
			$secret{$1} = $2;
	}
}
close C;

$CONFIG{'ADS'} = ['127.0.0.1:389'] unless exists $CONFIG{'ADS'};
$CONFIG{'PORT'} = [1812, 1813 ] unless exists $CONFIG{'PORT'};

my %svc = (
	'name'    =>  'SimpleRadius',
	'display' =>  'Simple Radius',
	'path'    =>  $^X,
	'user'    =>  '',
	'pwd'     =>  '',
	'description' =>
	"A simple RADIUS service for coupling Juniper/Netscreen devices to ADS.\n".
	"(C)2006-2007 by Marc Huber <Marc.Huber\@web.de>",
	'parameters' => '"' . $0 . '" -service',
);

my $timeout = undef;

foreach my $arg (@ARGV){
	if ($arg eq '-install'){
		if (Win32::Daemon::CreateService(\%svc)){
			print "Successfully added.\n";
		} else {
			print "Failed to add service: " .
				Win32::FormatMessage(Win32::Daemon::GetLastError()) . "\n";
		}
		exit 0;
	} elsif ($arg eq '-uninstall'){
		if (Win32::Daemon::DeleteService($svc{'name'})){
			print "Service was removed.\n";
		} else {
			print "Failed to remove service: " .
				Win32::FormatMessage(Win32::Daemon::GetLastError()) . "\n";
		}
		exit 0;
	} elsif ($arg eq '-service'){
		Win32::Daemon::AcceptedControls(
			SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN);
		Win32::Daemon::StartService();
		while(SERVICE_START_PENDING != Win32::Daemon::State()){ sleep 1; }
		Win32::Daemon::State(SERVICE_RUNNING);
		$timeout = 2;
	} elsif ($arg eq '-debug'){
		$debug = 1;
	} else {
		$0 =~ /([^\\]+)$/;
		print STDERR "Usage: $1 [ -install | -uninstall ]\n";
		exit 0;
	}
}

if (not $debug and $#ARGV < 0) {
	print STDERR $svc{'description'}, "\n\n", <<EOT

You may install this program as a standard service using the
    -install
switch, and you may uninstall the service using the
    -uninstall
argument.

Closing this window will terminate the program.
EOT
	;
}

my $dict = new RADIUS::Dictionary "<&DATA";
my %ATTR;

foreach my $a ([4, 'NS-Primary-DNS'], [5, 'NS-Secondary-DNS'],
	[6, 'NS-Primary-WINS'], [7, 'NS-Secondary-WINS']){
	$dict->{attr}->{$a->[1]} = [26, 'string'];
	$ATTR{$a->[1]} = pack('NC6', 3224, $a->[0], 6,
		split (/\./, $CONFIG{$a->[1]}->[0])) if exists $CONFIG{$a->[1]};
}

# For testing only ... doesn't seem to be evaluated on a SSG20 w/ 5.4.0r1
#$dict->{attr}->{'NS-Zone-Verification'} = [26, 'string'];
#$ATTR{'NS-Zone-Verification'} =
#	pack('NCCa*', 3224, 10, 2 + length ('Trusted'), 'Trusted');

$ATTR{'Framed-IP-Netmask'} = '255.255.255.255';

my $sel = IO::Select->new();

for my $port (@{$CONFIG{'PORT'}}){
	$sel->add(IO::Socket::INET->new(Proto => 'udp',
									LocalAddr => INADDR_ANY,
									LocalPort => $port,
									ReuseAddr => 1) or die "socket: $!");
}

while (1) {
	if (defined $timeout) {
		my $e = Win32::Daemon::QueryLastMessage(1);
		if ($e == SERVICE_CONTROL_STOP || $e == SERVICE_CONTROL_SHUTDOWN
			|| Win32::Daemon::State() != SERVICE_RUNNING) {
			Win32::Daemon::StopService();
			exit 0;
		} elsif ($e == SERVICE_CONTROL_INTERROGATE){
			Win32::Daemon::State(SERVICE_RUNNING);	
		}
	}

	foreach my $sock ($sel->can_read($timeout)) {
		my ($rec, $clientip, $sin, $req, $resp);
		$sin = $sock->recv($rec, 1500, 0) or next;
		$clientip = inet_ntoa((sockaddr_in($sin))[1]);
		exists $secret{$clientip} or next;

		$req = new RADIUS::Packet($dict, $rec);
		$resp = new RADIUS::Packet($dict);

		if ($req->code eq 'Access-Request') {
			my $user = lc $req->attr('User-Name');
			next unless $user =~ /^[0-9a-z_@%]+/;
			my $ldap = Net::LDAP->new(@{$CONFIG{'ADS'}});
#			$ldap->start_tls;
			my ($mesg, $domain) = (undef, undef);
			if ($user =~ /(.+)[\@%](.+)$/){
				my $d = $2;
				$user = $1;
				$mesg = $ldap->bind(
					$user,
					password => $req->password($secret{$clientip}),
					version => 3
				);
				$domain = $d unless $mesg->code;
			} else {
				foreach my $d (@{$CONFIG{'DOMAIN'}}){
					$mesg = $ldap->bind(
						$user . '@' . $d,
						password => $req->password($secret{$clientip}),
						version => 3
					);
					unless ($mesg->code){
						$domain = $d;
						last;
					}
				}
			}

			if (defined($mesg) && !$mesg->code){
				$mesg = $ldap->search(
					base => 'dc=' . join(',dc=', split(/\./, $domain)),
					scope => 'sub',
					attrs => ['memberof', 'msRADIUSFramedIPAddress'],
					filter => '(&(objectclass=user)(cn=' . $user . '))'
				);
				$mesg = undef unless $mesg->count == 1;
			    $ldap->unbind;
			} else {
				$mesg = undef;
			}
	
			if ($mesg){
				my $i = 0;
				foreach my $m ($mesg->entry(0)->get_value('memberof')) {
					if (exists $CONFIG{'GROUP-REGEX'}){
						my $r = join('|', @{$CONFIG{'GROUP-REGEX'}});
						next unless $m =~ /$r/i;
					}
					$dict->{'attr'}->{"NS-User-Group.$i"} = [26, 'string'];
					$m =~ s/^[^=]+=([^,]+),.*$/$1/;
					$m =~ s/\s+//g;
					$resp->set_attr("NS-User-Group.$i",
						pack('NCCa*', (3224, 3, 2 + length $m, lc $m)));
					$i++;
				}
				my $a = $mesg->entry(0)->get_value('msRADIUSFramedIPAddress');
				$resp->set_attr('Framed-IP-Address', sprintf ('%d.%d.%d.%d',
					$a >> 24, 0xff & ($a >> 16), 0xff & ($a >> 8), 0xff & $a))
					if defined $a;
				foreach my $k (keys %ATTR) { $resp->set_attr($k, $ATTR{$k}); }
				$resp->set_code('Access-Accept');
			} else {
				$resp->set_code('Access-Reject');
			}
		} elsif ($req->code eq 'Accounting-Request') {
			# FIXME -- could do some logging. For now it's sufficient
			# not to ignore accounting requests to keep xauth happy.
			$resp->set_code('Accounting-Response');
		} else {
				next;
		}
		$resp->set_identifier($req->identifier);
		$resp->set_authenticator($req->authenticator);
		if ($debug) { print scalar localtime, "\n"; $req->dump; $resp->dump; }
		$sock->send(auth_resp($resp->pack, $secret{$clientip}), 0, $sin);
	}
}

__DATA__

# Dictionary data generated from FreeRadius dictionary file by using
# egrep '^ *(ATTRIBUTE|VALUE)' | sed -r 's/(string|octet).*/string/g'
#

ATTRIBUTE	User-Name	1	string
ATTRIBUTE	User-Password	2	string
ATTRIBUTE	CHAP-Password	3	string
ATTRIBUTE	NAS-IP-Address	4	ipaddr
ATTRIBUTE	NAS-Port	5	integer
ATTRIBUTE	Service-Type	6	integer
ATTRIBUTE	Framed-Protocol	7	integer
ATTRIBUTE	Framed-IP-Address	8	ipaddr
ATTRIBUTE	Framed-IP-Netmask	9	ipaddr
ATTRIBUTE	Framed-Routing	10	integer
ATTRIBUTE	Filter-Id	11	string
ATTRIBUTE	Framed-MTU	12	integer
ATTRIBUTE	Framed-Compression	13	integer
ATTRIBUTE	Login-IP-Host	14	ipaddr
ATTRIBUTE	Login-Service	15	integer
ATTRIBUTE	Login-TCP-Port	16	integer
ATTRIBUTE	Reply-Message	18	string
ATTRIBUTE	Callback-Number	19	string
ATTRIBUTE	Callback-Id	20	string
ATTRIBUTE	Framed-Route	22	string
ATTRIBUTE	Framed-IPX-Network	23	ipaddr
ATTRIBUTE	State	24	string
ATTRIBUTE	Class	25	string
ATTRIBUTE	Vendor-Specific	26	string
ATTRIBUTE	Session-Timeout	27	integer
ATTRIBUTE	Idle-Timeout	28	integer
ATTRIBUTE	Termination-Action	29	integer
ATTRIBUTE	Called-Station-Id	30	string
ATTRIBUTE	Calling-Station-Id	31	string
ATTRIBUTE	NAS-Identifier	32	string
ATTRIBUTE	Proxy-State	33	string
ATTRIBUTE	Login-LAT-Service	34	string
ATTRIBUTE	Login-LAT-Node	35	string
ATTRIBUTE	Login-LAT-Group	36	string
ATTRIBUTE	Framed-AppleTalk-Link	37	integer
ATTRIBUTE	Framed-AppleTalk-Network	38	integer
ATTRIBUTE	Framed-AppleTalk-Zone	39	string
ATTRIBUTE	Acct-Status-Type	40	integer
ATTRIBUTE	Acct-Delay-Time	41	integer
ATTRIBUTE	Acct-Input-Octets	42	integer
ATTRIBUTE	Acct-Output-Octets	43	integer
ATTRIBUTE	Acct-Session-Id	44	string
ATTRIBUTE	Acct-Authentic	45	integer
ATTRIBUTE	Acct-Session-Time	46	integer
ATTRIBUTE	Acct-Input-Packets	47	integer
ATTRIBUTE	Acct-Output-Packets	48	integer
ATTRIBUTE	Acct-Terminate-Cause	49	integer
ATTRIBUTE	Acct-Multi-Session-Id	50	string
ATTRIBUTE	Acct-Link-Count	51	integer
ATTRIBUTE	Acct-Input-Gigawords	52	integer
ATTRIBUTE	Acct-Output-Gigawords	53	integer
ATTRIBUTE	Event-Timestamp	55	date
ATTRIBUTE	CHAP-Challenge	60	string
ATTRIBUTE	NAS-Port-Type	61	integer
ATTRIBUTE	Port-Limit	62	integer
ATTRIBUTE	Login-LAT-Port	63	integer
ATTRIBUTE	Acct-Tunnel-Connection	68	string
ATTRIBUTE	ARAP-Password	70	string
ATTRIBUTE	ARAP-Features	71	string
ATTRIBUTE	ARAP-Zone-Access	72	integer
ATTRIBUTE	ARAP-Security	73	integer
ATTRIBUTE	ARAP-Security-Data	74	string
ATTRIBUTE	Password-Retry	75	integer
ATTRIBUTE	Prompt	76	integer
ATTRIBUTE	Connect-Info	77	string
ATTRIBUTE	Configuration-Token	78	string
ATTRIBUTE	EAP-Message	79	string
ATTRIBUTE	Message-Authenticator	80	string
ATTRIBUTE	ARAP-Challenge-Response	84	string
ATTRIBUTE	Acct-Interim-Interval	85	integer
ATTRIBUTE	NAS-Port-Id	87	string
ATTRIBUTE	Framed-Pool	88	string
ATTRIBUTE	NAS-IPv6-Address	95	ipv6addr
ATTRIBUTE	Framed-Interface-Id	96	ifid
ATTRIBUTE	Framed-IPv6-Prefix	97	string
ATTRIBUTE	Login-IPv6-Host	98	ipv6addr
ATTRIBUTE	Framed-IPv6-Route	99	string
ATTRIBUTE	Framed-IPv6-Pool	100	string
ATTRIBUTE	Error-Cause	101	integer
ATTRIBUTE	Digest-Response	206	string
ATTRIBUTE	Digest-Attributes	207	string
ATTRIBUTE	Fall-Through	500	integer
ATTRIBUTE	Exec-Program	502	string
ATTRIBUTE	Exec-Program-Wait	503	string
ATTRIBUTE	Auth-Type	1000	integer
ATTRIBUTE	Menu	1001	string
ATTRIBUTE	Termination-Menu	1002	string
ATTRIBUTE	Prefix	1003	string
ATTRIBUTE	Suffix	1004	string
ATTRIBUTE	Group	1005	string
ATTRIBUTE	Crypt-Password	1006	string
ATTRIBUTE	Connect-Rate	1007	integer
ATTRIBUTE	Add-Prefix	1008	string
ATTRIBUTE	Add-Suffix	1009	string
ATTRIBUTE	Expiration	1010	date
ATTRIBUTE	Autz-Type	1011	integer
ATTRIBUTE	Acct-Type	1012	integer
ATTRIBUTE	Session-Type	1013	integer
ATTRIBUTE	Post-Auth-Type	1014	integer
ATTRIBUTE	Pre-Proxy-Type	1015	integer
ATTRIBUTE	Post-Proxy-Type	1016	integer
ATTRIBUTE	Pre-Acct-Type	1017	integer
ATTRIBUTE	EAP-Type	1018	integer
ATTRIBUTE	EAP-TLS-Require-Client-Cert	1019	integer
ATTRIBUTE	EAP-Id	1020	integer
ATTRIBUTE	EAP-Code	1021	integer
ATTRIBUTE	EAP-MD5-Password	1022	string
ATTRIBUTE	User-Category	1029	string
ATTRIBUTE	Group-Name	1030	string
ATTRIBUTE	Huntgroup-Name	1031	string
ATTRIBUTE	Simultaneous-Use	1034	integer
ATTRIBUTE	Strip-User-Name	1035	integer
ATTRIBUTE	Hint	1040	string
ATTRIBUTE	Pam-Auth	1041	string
ATTRIBUTE	Login-Time	1042	string
ATTRIBUTE	Stripped-User-Name	1043	string
ATTRIBUTE	Current-Time	1044	string
ATTRIBUTE	Realm	1045	string
ATTRIBUTE	No-Such-Attribute	1046	string
ATTRIBUTE	Packet-Type	1047	integer
ATTRIBUTE	Proxy-To-Realm	1048	string
ATTRIBUTE	Replicate-To-Realm	1049	string
ATTRIBUTE	Acct-Session-Start-Time	1050	date
ATTRIBUTE	Acct-Unique-Session-Id	1051	string
ATTRIBUTE	Client-IP-Address	1052	ipaddr
ATTRIBUTE	Ldap-UserDn	1053	string
ATTRIBUTE	NS-MTA-MD5-Password	1054	string
ATTRIBUTE	SQL-User-Name	1055	string
ATTRIBUTE	LM-Password	1057	string
ATTRIBUTE	NT-Password	1058	string
ATTRIBUTE	SMB-Account-CTRL	1059	integer
ATTRIBUTE	SMB-Account-CTRL-TEXT	1061	string
ATTRIBUTE	User-Profile	1062	string
ATTRIBUTE	Digest-Realm	1063	string
ATTRIBUTE	Digest-Nonce	1064	string
ATTRIBUTE	Digest-Method	1065	string
ATTRIBUTE	Digest-URI	1066	string
ATTRIBUTE	Digest-QOP	1067	string
ATTRIBUTE	Digest-Algorithm	1068	string
ATTRIBUTE	Digest-Body-Digest	1069	string
ATTRIBUTE	Digest-CNonce	1070	string
ATTRIBUTE	Digest-Nonce-Count	1071	string
ATTRIBUTE	Digest-User-Name	1072	string
ATTRIBUTE	Pool-Name	1073	string
ATTRIBUTE	Ldap-Group	1074	string
ATTRIBUTE	Module-Success-Message	1075	string
ATTRIBUTE	Module-Failure-Message	1076	string
ATTRIBUTE	Rewrite-Rule	1078	string
ATTRIBUTE	Sql-Group	1079	string
ATTRIBUTE	Response-Packet-Type	1080	integer
ATTRIBUTE	Packet-Dst-Port	1081	integer
ATTRIBUTE	MS-CHAP-Use-NTLM-Auth	1082	integer
ATTRIBUTE	NTLM-User-Name	1083	string
ATTRIBUTE	EAP-Sim-Subtype	1200	integer
ATTRIBUTE	EAP-Sim-Rand1	1201	string
ATTRIBUTE	EAP-Sim-Rand2	1202	string
ATTRIBUTE	EAP-Sim-Rand3	1203	string
ATTRIBUTE	EAP-Sim-SRES1	1204	string
ATTRIBUTE	EAP-Sim-SRES2	1205	string
ATTRIBUTE	EAP-Sim-SRES3	1206	string
VALUE	EAP-Sim-Subtype	Start	10
VALUE	EAP-Sim-Subtype	Challenge	11
VALUE	EAP-Sim-Subtype	Notification	12
VALUE	EAP-Sim-Subtype	Re-authentication	13
ATTRIBUTE	EAP-Sim-State	1207	integer
ATTRIBUTE	EAP-Sim-IMSI	1208	string
ATTRIBUTE	EAP-Sim-HMAC	1209	string
ATTRIBUTE	EAP-Sim-KEY	1210	string
ATTRIBUTE	EAP-Sim-EXTRA	1211	string
ATTRIBUTE	EAP-Sim-KC1	1212	string
ATTRIBUTE	EAP-Sim-KC2	1213	string
ATTRIBUTE	EAP-Sim-KC3	1214	string
ATTRIBUTE	EAP-Type-Identity	1281	string
ATTRIBUTE	EAP-Type-Notification	1282	string
ATTRIBUTE	EAP-Type-NAK	1283	string
ATTRIBUTE	EAP-Type-MD5	1284	string
ATTRIBUTE	EAP-Type-OTP	1285	string
ATTRIBUTE	EAP-Type-GTC	1286	string
ATTRIBUTE	EAP-Type-TLS	1297	string
ATTRIBUTE	EAP-Type-SIM	1298	string
ATTRIBUTE	EAP-Type-LEAP	1301	string
ATTRIBUTE	EAP-Type-SIM2	1302	string
ATTRIBUTE	EAP-Type-TTLS	1305	string
ATTRIBUTE	EAP-Type-PEAP	1309	string
ATTRIBUTE	EAP-Sim-RAND	1537	string
ATTRIBUTE	EAP-Sim-PADDING	1542	string
ATTRIBUTE	EAP-Sim-NONCE_MT	1543	string
ATTRIBUTE	EAP-Sim-PERMANENT_ID_REQ	1546	string
ATTRIBUTE	EAP-Sim-MAC	1547	string
ATTRIBUTE	EAP-Sim-NOTIFICATION	1548	string
ATTRIBUTE	EAP-Sim-ANY_ID_REQ	1549	string
ATTRIBUTE	EAP-Sim-IDENTITY	1550	string
ATTRIBUTE	EAP-Sim-VERSION_LIST	1551	string
ATTRIBUTE	EAP-Sim-SELECTED_VERSION	1552	string
ATTRIBUTE	EAP-Sim-FULLAUTH_ID_REQ	1553	string
ATTRIBUTE	EAP-Sim-COUNTER	1555	string
ATTRIBUTE	EAP-Sim-COUNTER_TOO_SMALL	1556	string
ATTRIBUTE	EAP-Sim-NONCE_S	1557	string
ATTRIBUTE	EAP-Sim-IV	1665	string
ATTRIBUTE	EAP-Sim-ENCR_DATA	1666	string
ATTRIBUTE	EAP-Sim-NEXT_PSEUDONUM	1668	string
ATTRIBUTE	EAP-Sim-NEXT_REAUTH_ID	1669	string
ATTRIBUTE	EAP-Sim-CHECKCODE	1670	string
VALUE	Service-Type	Login-User	1
VALUE	Service-Type	Framed-User	2
VALUE	Service-Type	Callback-Login-User	3
VALUE	Service-Type	Callback-Framed-User	4
VALUE	Service-Type	Outbound-User	5
VALUE	Service-Type	Administrative-User	6
VALUE	Service-Type	NAS-Prompt-User	7
VALUE	Service-Type	Authenticate-Only	8
VALUE	Service-Type	Callback-NAS-Prompt	9
VALUE	Service-Type	Call-Check	10
VALUE	Service-Type	Callback-Administrative	11
VALUE	Service-Type	Voice	12
VALUE	Service-Type	Fax	13
VALUE	Service-Type	Modem-Relay	14
VALUE	Service-Type	IAPP-Register	15
VALUE	Service-Type	IAPP-AP-Check	16
VALUE	Framed-Protocol	PPP	1
VALUE	Framed-Protocol	SLIP	2
VALUE	Framed-Protocol	ARAP	3
VALUE	Framed-Protocol	Gandalf-SLML	4
VALUE	Framed-Protocol	Xylogics-IPX-SLIP	5
VALUE	Framed-Protocol	X.75-Synchronous	6
VALUE	Framed-Protocol	GPRS-PDP-Context	7
VALUE	Framed-Routing	None	0
VALUE	Framed-Routing	Broadcast	1
VALUE	Framed-Routing	Listen	2
VALUE	Framed-Routing	Broadcast-Listen	3
VALUE	Framed-Compression	None	0
VALUE	Framed-Compression	Van-Jacobson-TCP-IP	1
VALUE	Framed-Compression	IPX-Header-Compression	2
VALUE	Framed-Compression	Stac-LZS	3
VALUE	Login-Service	Telnet	0
VALUE	Login-Service	Rlogin	1
VALUE	Login-Service	TCP-Clear	2
VALUE	Login-Service	PortMaster	3
VALUE	Login-Service	LAT	4
VALUE	Login-Service	X25-PAD	5
VALUE	Login-Service	X25-T3POS	6
VALUE	Login-Service	TCP-Clear-Quiet	7
VALUE	Login-TCP-Port	Telnet	23
VALUE	Login-TCP-Port	Rlogin	513
VALUE	Login-TCP-Port	Rsh	514
VALUE	Acct-Status-Type	Start	1
VALUE	Acct-Status-Type	Stop	2
VALUE	Acct-Status-Type	Interim-Update	3
VALUE	Acct-Status-Type	Alive	3
VALUE	Acct-Status-Type	Accounting-On	7
VALUE	Acct-Status-Type	Accounting-Off	8
VALUE	Acct-Status-Type	Tunnel-Start	9
VALUE	Acct-Status-Type	Tunnel-Stop	10
VALUE	Acct-Status-Type	Tunnel-Reject	11
VALUE	Acct-Status-Type	Tunnel-Link-Start	12
VALUE	Acct-Status-Type	Tunnel-Link-Stop	13
VALUE	Acct-Status-Type	Tunnel-Link-Reject	14
VALUE	Acct-Status-Type	Failed	15
VALUE	Acct-Authentic	RADIUS	1
VALUE	Acct-Authentic	Local	2
VALUE	Acct-Authentic	Remote	3
VALUE	Acct-Authentic	Diameter	4
VALUE	Termination-Action	Default	0
VALUE	Termination-Action	RADIUS-Request	1
VALUE	NAS-Port-Type	Async	0
VALUE	NAS-Port-Type	Sync	1
VALUE	NAS-Port-Type	ISDN	2
VALUE	NAS-Port-Type	ISDN-V120	3
VALUE	NAS-Port-Type	ISDN-V110	4
VALUE	NAS-Port-Type	Virtual	5
VALUE	NAS-Port-Type	PIAFS	6
VALUE	NAS-Port-Type	HDLC-Clear-Channel	7
VALUE	NAS-Port-Type	X.25	8
VALUE	NAS-Port-Type	X.75	9
VALUE	NAS-Port-Type	G.3-Fax	10
VALUE	NAS-Port-Type	SDSL	11
VALUE	NAS-Port-Type	ADSL-CAP	12
VALUE	NAS-Port-Type	ADSL-DMT	13
VALUE	NAS-Port-Type	IDSL	14
VALUE	NAS-Port-Type	Ethernet	15
VALUE	NAS-Port-Type	xDSL	16
VALUE	NAS-Port-Type	Cable	17
VALUE	NAS-Port-Type	Wireless-Other	18
VALUE	NAS-Port-Type	Wireless-802.11	19
VALUE	NAS-Port-Type	Token-Ring	20
VALUE	NAS-Port-Type	FDDI	21
VALUE	NAS-Port-Type	Wireless-CDMA2000	22
VALUE	NAS-Port-Type	Wireless-UMTS	23
VALUE	NAS-Port-Type	Wireless-1X-EV	24
VALUE	NAS-Port-Type	IAPP	25
VALUE	Acct-Terminate-Cause	User-Request	1
VALUE	Acct-Terminate-Cause	Lost-Carrier	2
VALUE	Acct-Terminate-Cause	Lost-Service	3
VALUE	Acct-Terminate-Cause	Idle-Timeout	4
VALUE	Acct-Terminate-Cause	Session-Timeout	5
VALUE	Acct-Terminate-Cause	Admin-Reset	6
VALUE	Acct-Terminate-Cause	Admin-Reboot	7
VALUE	Acct-Terminate-Cause	Port-Error	8
VALUE	Acct-Terminate-Cause	NAS-Error	9
VALUE	Acct-Terminate-Cause	NAS-Request	10
VALUE	Acct-Terminate-Cause	NAS-Reboot	11
VALUE	Acct-Terminate-Cause	Port-Unneeded	12
VALUE	Acct-Terminate-Cause	Port-Preempted	13
VALUE	Acct-Terminate-Cause	Port-Suspended	14
VALUE	Acct-Terminate-Cause	Service-Unavailable	15
VALUE	Acct-Terminate-Cause	Callback	16
VALUE	Acct-Terminate-Cause	User-Error	17
VALUE	Acct-Terminate-Cause	Host-Request	18
VALUE	Acct-Terminate-Cause	Supplicant-Restart	19
VALUE	Acct-Terminate-Cause	Reauthentication-Failure	20
VALUE	Acct-Terminate-Cause	Port-Reinit	21
VALUE	Acct-Terminate-Cause	Port-Disabled	22
VALUE	Prompt	No-Echo	0
VALUE	Prompt	Echo	1
VALUE	Error-Cause	Residual-Context-Removed	201
VALUE	Error-Cause	Invalid-EAP-Packet	202
VALUE	Error-Cause	Unsupported-Attribute	401
VALUE	Error-Cause	Missing-Attribute	402
VALUE	Error-Cause	NAS-Identification-Mismatch	403
VALUE	Error-Cause	Invalid-Request	404
VALUE	Error-Cause	Unsupported-Service	405
VALUE	Error-Cause	Unsupported-Extension	406
VALUE	Error-Cause	Administratively-Prohibited	501
VALUE	Error-Cause	Proxy-Request-Not-Routable	502
VALUE	Error-Cause	Session-Context-Not-Found	503
VALUE	Error-Cause	Session-Context-Not-Removable	504
VALUE	Error-Cause	Proxy-Processing-Error	505
VALUE	Error-Cause	Resources-Unavailable	506
VALUE	Error-Cause	Request-Initiated	507
VALUE	Auth-Type	Local	0
VALUE	Auth-Type	System	1
VALUE	Auth-Type	SecurID	2
VALUE	Auth-Type	Crypt-Local	3
VALUE	Auth-Type	Reject	4
VALUE	Auth-Type	ActivCard	5
VALUE	Auth-Type	EAP	6
VALUE	Auth-Type	ARAP	7
VALUE	Auth-Type	Accept	254
VALUE	Auth-Type	PAP	1024
VALUE	Auth-Type	CHAP	1025
VALUE	Auth-Type	LDAP	1026
VALUE	Auth-Type	PAM	1027
VALUE	Auth-Type	MS-CHAP	1028
VALUE	Auth-Type	Kerberos	1029
VALUE	Auth-Type	CRAM	1030
VALUE	Auth-Type	NS-MTA-MD5	1031
VALUE	Auth-Type	SMB	1033
VALUE	Autz-Type	Local	0
VALUE	Acct-Type	Local	0
VALUE	Session-Type	Local	0
VALUE	Post-Auth-Type	Local	0
VALUE	Fall-Through	No	0
VALUE	Fall-Through	Yes	1
VALUE	Packet-Type	Access-Request	1
VALUE	Packet-Type	Access-Accept	2
VALUE	Packet-Type	Access-Reject	3
VALUE	Packet-Type	Accounting-Request	4
VALUE	Packet-Type	Accounting-Response	5
VALUE	Packet-Type	Accounting-Status	6
VALUE	Packet-Type	Password-Request	7
VALUE	Packet-Type	Password-Accept	8
VALUE	Packet-Type	Password-Reject	9
VALUE	Packet-Type	Accounting-Message	10
VALUE	Packet-Type	Access-Challenge	11
VALUE	Packet-Type	Status-Server	12
VALUE	Packet-Type	Status-Client	13
VALUE	Packet-Type	Disconnect-Request	40
VALUE	Packet-Type	Disconnect-ACK	41
VALUE	Packet-Type	Disconnect-NAK	42
VALUE	Packet-Type	CoA-Request	43
VALUE	Packet-Type	CoA-ACK	44
VALUE	Packet-Type	CoA-NAK	45
VALUE	Packet-Type	Resource-Free-Request	21
VALUE	Packet-Type	Resource-Free-Response	22
VALUE	Packet-Type	Resource-Query-Request	23
VALUE	Packet-Type	Resource-Query-Response	24
VALUE	Packet-Type	Alternate-Resource-Reclaim-Request	25
VALUE	Packet-Type	NAS-Reboot-Request	26
VALUE	Packet-Type	NAS-Reboot-Response	27
VALUE	Packet-Type	Next-Passcode	29
VALUE	Packet-Type	New-Pin	30
VALUE	Packet-Type	Terminate-Session	31
VALUE	Packet-Type	Password-Expired	32
VALUE	Packet-Type	Event-Request	33
VALUE	Packet-Type	Event-Response	34
VALUE	Packet-Type	Disconnect-Request	40
VALUE	Packet-Type	Disconnect-ACK	41
VALUE	Packet-Type	Disconnect-NAK	42
VALUE	Packet-Type	CoF-Request	43
VALUE	Packet-Type	CoF-ACK	44
VALUE	Packet-Type	CoF-NAK	45
VALUE	Packet-Type	IP-Address-Allocate	50
VALUE	Packet-Type	IP-Address-Release	51
VALUE	Response-Packet-Type	Access-Request	1
VALUE	Response-Packet-Type	Access-Accept	2
VALUE	Response-Packet-Type	Access-Reject	3
VALUE	Response-Packet-Type	Accounting-Request	4
VALUE	Response-Packet-Type	Accounting-Response	5
VALUE	Response-Packet-Type	Accounting-Status	6
VALUE	Response-Packet-Type	Password-Request	7
VALUE	Response-Packet-Type	Password-Accept	8
VALUE	Response-Packet-Type	Password-Reject	9
VALUE	Response-Packet-Type	Accounting-Message	10
VALUE	Response-Packet-Type	Access-Challenge	11
VALUE	Response-Packet-Type	Status-Server	12
VALUE	Response-Packet-Type	Status-Client	13
VALUE	EAP-Type	None	0
VALUE	EAP-Type	Identity	1
VALUE	EAP-Type	Notification	2
VALUE	EAP-Type	NAK	3
VALUE	EAP-Type	MD5-Challenge	4
VALUE	EAP-Type	One-Time-Password	5
VALUE	EAP-Type	Generic-Token-Card	6
VALUE	EAP-Type	RSA-Public-Key	9
VALUE	EAP-Type	DSS-Unilateral	10
VALUE	EAP-Type	KEA	11
VALUE	EAP-Type	KEA-Validate	12
VALUE	EAP-Type	KEA	11
VALUE	EAP-Type	EAP-TLS	13
VALUE	EAP-Type	Defender-Token	14
VALUE	EAP-Type	RSA-SecurID-EAP	15
VALUE	EAP-Type	Arcot-Systems-EAP	16
VALUE	EAP-Type	Cisco-LEAP	17
VALUE	EAP-Type	Nokia-IP-Smart-Card	18
VALUE	EAP-Type	SIM	18
VALUE	EAP-Type	SRP-SHA1-Part-1	19
VALUE	EAP-Type	SRP-SHA1-Part-2	20
VALUE	EAP-Type	EAP-TTLS	21
VALUE	EAP-Type	Remote-Access-Service	22
VALUE	EAP-Type	UMTS	23
VALUE	EAP-Type	EAP-3Com-Wireless	24
VALUE	EAP-Type	PEAP	25
VALUE	EAP-Type	MS-EAP-Authentication	26
VALUE	EAP-Type	MAKE	27
VALUE	EAP-Type	CRYPTOCard	28
VALUE	EAP-Type	EAP-MSCHAP-V2	29
VALUE	EAP-Type	DynamID	30
VALUE	EAP-Type	Rob-EAP	31
VALUE	EAP-Type	SecurID-EAP	32
VALUE	EAP-Type	MS-Authentication-TLV	33
VALUE	EAP-Type	SentriNET	34
VALUE	EAP-Type	EAP-Actiontec-Wireless	35
VALUE	EAP-Type	Cogent-Biomentric-EAP	36
VALUE	EAP-Type	AirFortress-EAP	37
VALUE	EAP-Type	EAP-HTTP-Digest	38
VALUE	EAP-Type	SecuriSuite-EAP	39
VALUE	EAP-Type	DeviceConnect-EAP	40
VALUE	EAP-Type	EAP-SPEKE	41
VALUE	EAP-Type	EAP-MOBAC	42
VALUE	EAP-Type	Microsoft-MS-CHAPv2	26
VALUE	EAP-Type	Cisco-MS-CHAPv2	29
VALUE	EAP-Type	MS-CHAP-V2	26
VALUE	EAP-TLS-Require-Client-Cert	No	0
VALUE	EAP-TLS-Require-Client-Cert	Yes	1
VALUE	EAP-Code	Request	1
VALUE	EAP-Code	Response	2
VALUE	EAP-Code	Success	3
VALUE	EAP-Code	Failure	4
VALUE	MS-CHAP-Use-NTLM-Auth	No	0
VALUE	MS-CHAP-Use-NTLM-Auth	Yes	1

# vim:ts=4
